'use strict'
# 그리기 영역의 정보 
margin = new Margin(30, 50)
[width, height] = [800, 500]

#------------------------------------------------------------
#
# ## 꺾은선그래프를 그리는 함수 
#
# * @param  target        문자열 svg를 추가하는 요소의 XPath
# * @param  data          배열   데이터의 배열 
# * @param  info          문자열 서버정보의 이름(CPU 등)
# * @param  opt           높이와 폭, 마진, 타이틀 등의 정보
# * @return d3.selection  chart를 그린 svg요소 
#
drawChart = (target, data, info, opt)->
  # svg요소 작성 
  svg = d3.select(target).append('svg')
    .attr(width:  opt.width  + opt.margin.width, height: opt.height + opt.margin.height)
  main = svg
    .append('g')
    .attr(
      width: opt.width, height: opt.height
      transform: "translate(#{opt.margin.left},#{opt.margin.top})"
    )

  # 타이틀이 있으면 text 요소를 작성 
  if opt.title
    svg.append('text').classed('chart-title', true).text(opt.title)
      .attr(dx: '1em', dy: '1.25em')

  # 시간의 폭, 서버정보의 범위를 가져온다 
  xExtent = d3.extent data, (d)-> new Date(d.epoch * 1000)
  yExtent = d3.extent data, (d)-> +d[info]

  # x축, y축 색 스케일을 작성 
  x = d3.time.scale().range([0, opt.width]).domain(xExtent)
  y = d3.scale.linear().range([opt.height, 0]).domain([0, yExtent[1]])
  color = d3.scale.category10()

  # 선분의 path 요소의 d속성값을 계산한다 
  line = d3.svg.line().x((d)-> x(new Date(d.epoch * 1000))).y((d)-> y(+d[info]))

  # path 속성을 추가해서 그린다 
  path = main.append('path').datum(data).attr(
    class: 'line'
    d: line
    fill: 'none'
    stroke: color(0)
  )

  # 축을 작성 
  xaxis = d3.svg.axis().scale(x).ticks(6).tickFormat d3.time.format('%H:%M')
  yaxis = d3.svg.axis().orient('left').scale(y).tickFormat(d3.format('s')).ticks(4)
  xAxis = main.append('g').classed('axis', true).call(xaxis)
    .attr('transform', "translate(0, #{opt.height})")
  yAxis = main.append('g').classed('axis', true).call(yaxis)

  # 이벤트 핸들러 작성 
  brushend = ()->
    # 애니메이션을 멈추기 위한 정보로서 클래스를 지정 
    svg.classed('brushing', not brush.empty())
    # brush의 범위를 가져온다. 단, empty일 때는 리셋으로 간주해서 데이터의 모든 범위로 한다.
    extent = if brush.empty() then xExtent else brush.extent()
    # 스케일 변경 
    x.domain(extent)
    # 축 변경 
    xAxis.call(xaxis)
    # path의 d속성 변경. brush의 범위 내의 데이터만으로 선분군을 작성 
    path.attr('d', line(data.filter((d)-> extent[0] < new Date(d.epoch * 1000) < extent[1])))

    # 이번 케이스에서는 선택한 범위가 최대로 확대되어 범위이동 등은 없기에 brush의 음영삭제 
    brush.clear()
    gBrush.select('.extent').attr('width', 0)

  brush = d3.svg.brush().x(x).on('brushend', brushend)

  # brush를 적용할 group요소 생성 
  gBrush = main.append('g').call(brush)
  gBrush.selectAll("rect")
    .attr("height", height)
    .style('opacity', .125)
    .style('shape-rendering', 'crispEdges')

  svg.update = ()->
    # 데이터가 2개 이하의 경우에는 아무것도 하지 않는다 
    return if data.length < 2
    if not svg.classed('brushing')
      # 데이터의 범위를 갱신 
      xExtent = d3.extent data, (d)-> new Date(d.epoch * 1000)
      yExtent = d3.extent data, (d)-> +d[info]

      # 스케일의 도메인 (데이터의 범위)를 갱신 
      x.domain(xExtent)
      y.domain([0, yExtent[1]])

      brush.extent(xExtent)

      # transition()을 작성하고 변경 후의 데이터를 설정 
      path.transition().attr('d', line)

      # 축의 갱신 
      xAxis.call(xaxis)
      yAxis.call(yaxis)

  return svg

#------------------------------------------------------------
#
# ## dstat의 헤더로부터 서버정보 목록을 가져온다 
#
# * @param  headers 문자열 서버정보 
# * @return [hostInfo infoCategoryMap, infoLabel]
#   * hostInfo 호스트 정보, infoCategoryMap 상세 서버정보와 카테고리의 대응, infoLabels 서버정보의 라벨 
#
parseHeader = (headers)->
  hostInfo = headers[2].split(/,/).map (d)-> d.replace(/^\"|\"$/g, "")
  categoryLabels = headers[5].split(/;/)
  infoLabels  = headers[6].split(/;/)
  infoCategoryMap = {}

  # \"를 트림 
  for categoryLabel, idx in categoryLabels
    categoryLabels[idx] = categoryLabel.replace(/^\"|\"$/g, "").replace(/\s/g, '_')
  for infoLabel, idx in infoLabels
    infoLabels[idx] = infoLabel.replace(/^\"|\"$/g, "").replace(/\s/g, '_')

  # 카테고리로 분류 / 상세 라벨 취득 
  currentCategory = null
  for infoLabel, idx in infoLabels
    if categoryLabels[idx] isnt ""
      currentCategory = categoryLabels[idx]
    infoCategoryMap[currentCategory] = [] unless infoCategoryMap[currentCategory]
    infoCategoryMap[currentCategory].push infoLabel

    if infoLabel is currentCategory
      continue
    infoLabels[idx] = "#{currentCategory}.#{infoLabel}"

  [hostInfo[1], infoCategoryMap, infoLabels]

#------------------------------------------------------------
#
# # dstat.log를 가져와서, 1초에 하나씩 지정한 서버정보 데이터를 추가한 꺽은선 그래프를 그린다
#
$.get("dstat.log").done (log)->
  # 세미콜론 separated values の parserを作成
  semicolonSv = d3.dsv(";", "text/ssv")
  # 1행 1레코드로 분할
  lines = log.split(/\n/)
  # ヘッダを parse
  [host, infoCategoryMap, infoLabels] = parseHeader(lines.slice(0, 7))

  lines = lines.slice(7)
  # 레코드를 해석하여 JSON 데이터로 변환 
  alldata = semicolonSv.parse(infoLabels.join(';') + "\n" + lines.join('\n'))
  data = []

  # 아래 정보를 표시한다 
  infos = ["net/total.recv", "net/total.send", "dsk/total.read", "dsk/total.writ", "io/total.read", "io/total.writ"]
  svgs = []
  for info in infos
    # 그래프를 그린다 
    svgs.push drawChart("body", data, info, {
      width: width
      height: 60
      margin: margin
      title: info
      })

  # 매초 데이터를 하나 가져와서 반영한다 
  timer = setInterval ()->
    if alldata.length is 0
      return clearInterval timer
    data.push alldata.shift()
    for svg in svgs
      svg.update()
  , 1000
